@@ -29,7 +29,6 @@ use sixtyfps_corelib as corelib;
 use winit::dpi::LogicalSize;
 
 use crate::CanvasRc;
-use crate::ItemGraphicsCacheEntry;
 
 /// GraphicsWindow is an implementation of the [PlatformWindow][`crate::eventloop::PlatformWindow`] trait. This is
 /// typically instantiated by entry factory functions of the different graphics back ends.
@@ -434,7 +433,16 @@ impl PlatformWindow for GraphicsWindow {
         if h > 0. {
             size.height = h as _;
         }
-        self.borrow_mapped_window().unwrap().opengl_context.window().set_inner_size(size);
+        {
+            let mapped_window = self.borrow_mapped_window().unwrap();
+            let winit_window = mapped_window.opengl_context.window();
+            // If we're in fullscreen state, don't try to resize the window but maintain the surface
+            // size we've been assigned to from the windowing system. Weston/Wayland don't like it
+            // when we create a surface that's bigger than the screen due to constraints (#532).
+            if winit_window.fullscreen().is_none() {
+                winit_window.set_inner_size(size);
+            }
+        }
         if must_resize {
             self.set_geometry(size.width as _, size.height as _)
         }
@@ -446,29 +454,33 @@ impl PlatformWindow for GraphicsWindow {
         constraints_vertical: corelib::layout::LayoutInfo,
     ) {
         if let Some(window) = self.borrow_mapped_window() {
+            let winit_window = window.opengl_context.window();
+            // If we're in fullscreen state, don't try to resize the window but maintain the surface
+            // size we've been assigned to from the windowing system. Weston/Wayland don't like it
+            // when we create a surface that's bigger than the screen due to constraints (#532).
+            if winit_window.fullscreen().is_some() {
+                return;
+            }
+
             if (constraints_horizontal, constraints_vertical) != window.constraints.get() {
                 let min_width = constraints_horizontal.min.min(constraints_horizontal.max);
                 let min_height = constraints_vertical.min.min(constraints_vertical.max);
                 let max_width = constraints_horizontal.max.max(constraints_horizontal.min);
                 let max_height = constraints_vertical.max.max(constraints_vertical.min);
 
-                window.opengl_context.window().set_min_inner_size(
-                    if min_width > 0. || min_height > 0. {
-                        Some(winit::dpi::LogicalSize::new(min_width, min_height))
-                    } else {
-                        None
-                    },
-                );
-                window.opengl_context.window().set_max_inner_size(
-                    if max_width < f32::MAX || max_height < f32::MAX {
-                        Some(winit::dpi::LogicalSize::new(
-                            max_width.min(65535.),
-                            max_height.min(65535.),
-                        ))
-                    } else {
-                        None
-                    },
-                );
+                winit_window.set_min_inner_size(if min_width > 0. || min_height > 0. {
+                    Some(winit::dpi::LogicalSize::new(min_width, min_height))
+                } else {
+                    None
+                });
+                winit_window.set_max_inner_size(if max_width < f32::MAX || max_height < f32::MAX {
+                    Some(winit::dpi::LogicalSize::new(
+                        max_width.min(65535.),
+                        max_height.min(65535.),
+                    ))
+                } else {
+                    None
+                });
                 window.constraints.set((constraints_horizontal, constraints_vertical));
 
                 #[cfg(target_arch = "wasm32")]
@@ -505,19 +517,15 @@ impl PlatformWindow for GraphicsWindow {
 
     fn text_size(
         &self,
-        item_graphics_cache: &corelib::item_rendering::CachedRenderingData,
-        unresolved_font_request_getter: &dyn Fn() -> corelib::graphics::FontRequest,
+        font_request: corelib::graphics::FontRequest,
         text: &str,
         max_width: Option<f32>,
     ) -> Size {
-        let font_request_fn =
-            || unresolved_font_request_getter().merge(&self.default_font_properties());
+        let font_request = font_request.merge(&self.default_font_properties());
 
         crate::fonts::text_size(
-            &self.graphics_cache,
-            item_graphics_cache,
-            font_request_fn,
-            self.self_weak.upgrade().unwrap().scale_factor_property(),
+            &font_request,
+            self.self_weak.upgrade().unwrap().scale_factor(),
             text,
             max_width,
         )
@@ -540,20 +548,13 @@ impl PlatformWindow for GraphicsWindow {
             return 0;
         }
 
-        let font = text_input
-            .cached_rendering_data
-            .get_or_update(&self.graphics_cache, || {
-                Some(ItemGraphicsCacheEntry::Font(crate::fonts::FONT_CACHE.with(|cache| {
-                    cache.borrow_mut().font(
-                        text_input.unresolved_font_request().merge(&self.default_font_properties()),
-                        scale_factor,
-                        &text_input.text(),
-                    )
-                })))
-            })
-            .unwrap()
-            .as_font()
-            .clone();
+        let font = crate::fonts::FONT_CACHE.with(|cache| {
+            cache.borrow_mut().font(
+                text_input.unresolved_font_request().merge(&self.default_font_properties()),
+                scale_factor,
+                &text_input.text(),
+            )
+        });
 
         let paint = font.init_paint(text_input.letter_spacing() * scale_factor, Default::default());
         let text_context =
@@ -602,20 +603,13 @@ impl PlatformWindow for GraphicsWindow {
             return result;
         }
 
-        let font = text_input
-            .cached_rendering_data
-            .get_or_update(&self.graphics_cache, || {
-                Some(ItemGraphicsCacheEntry::Font(crate::fonts::FONT_CACHE.with(|cache| {
-                    cache.borrow_mut().font(
-                        text_input.unresolved_font_request().merge(&self.default_font_properties()),
-                        scale_factor,
-                        &text_input.text(),
-                    )
-                })))
-            })
-            .unwrap()
-            .as_font()
-            .clone();
+        let font = crate::fonts::FONT_CACHE.with(|cache| {
+            cache.borrow_mut().font(
+                text_input.unresolved_font_request().merge(&self.default_font_properties()),
+                scale_factor,
+                &text_input.text(),
+            )
+        });
 
         let paint = font.init_paint(text_input.letter_spacing() * scale_factor, Default::default());
         crate::fonts::layout_text_lines(
