@@ -28,7 +28,7 @@ use sixtyfps_corelib::graphics::{Brush, Color, Image, IntRect, Point, Rect, Rend
 use sixtyfps_corelib::item_rendering::{CachedRenderingData, ItemRenderer};
 use sixtyfps_corelib::items::{FillRule, ImageFit, ImageRendering};
 use sixtyfps_corelib::properties::Property;
-use sixtyfps_corelib::window::Window;
+use sixtyfps_corelib::window::{Window, WindowRc};
 
 mod graphics_window;
 use graphics_window::*;
@@ -50,11 +50,9 @@ enum ItemGraphicsCacheEntry {
     ColorizedImage {
         // This original image Rc is kept here to keep the image in the shared image cache, so that
         // changes to the colorization brush will not require re-uploading the image.
-        original_image: Rc<CachedImage>,
+        _original_image: Rc<CachedImage>,
         colorized_image: Rc<CachedImage>,
     },
-    // The font selection is expensive because it is also based on the concrete rendered text, so this is cached here to speed up re-paints
-    Font(fonts::Font),
 }
 
 impl ItemGraphicsCacheEntry {
@@ -62,18 +60,11 @@ impl ItemGraphicsCacheEntry {
         match self {
             ItemGraphicsCacheEntry::Image(image) => image,
             ItemGraphicsCacheEntry::ColorizedImage { colorized_image, .. } => colorized_image,
-            _ => panic!("internal error. image requested for non-image gpu data"),
         }
     }
     fn is_colorized_image(&self) -> bool {
         matches!(self, ItemGraphicsCacheEntry::ColorizedImage { .. })
     }
-    fn as_font(&self) -> &fonts::Font {
-        match self {
-            ItemGraphicsCacheEntry::Font(font) => font,
-            _ => panic!("internal error. font requested for non-font gpu data"),
-        }
-    }
 }
 
 type ItemGraphicsCache = RenderingCache<Option<ItemGraphicsCacheEntry>>;
@@ -169,14 +160,68 @@ impl OpenGLContext {
     ) -> (Self, femtovg::renderer::OpenGl) {
         #[cfg(not(target_arch = "wasm32"))]
         {
+            use crate::event_loop::EventLoopInterface;
+            use glutin::ContextBuilder;
             let windowed_context = crate::event_loop::with_window_target(|event_loop| {
-                let builder = glutin::ContextBuilder::new().with_vsync(true);
+                // Try different strategies for creating an GL context. First request our "favorite", OpenGL ES 2.0,
+                // then try GlLatest (but with windows quirk) and finally try glutin's defaults.
+                // We might be able to just go back to requesting GlLatest if
+                // https://github.com/rust-windowing/glutin/issues/1371 is resolved
+                // in favor of falling back to creating a GLES context.
+                let context_factory_fns = [
+                    |window_builder, event_loop: &dyn EventLoopInterface| {
+                        let builder = ContextBuilder::new()
+                            .with_vsync(true)
+                            .with_gl(glutin::GlRequest::Specific(glutin::Api::OpenGlEs, (2, 0)));
+                        #[cfg(target_os = "windows")]
+                        let builder = builder.with_srgb(false);
+                        builder
+                            .build_windowed(window_builder, event_loop.event_loop_target())
+                            .map_err(|creation_error| {
+                                format!(
+                                    "could not create OpenGL ES 2.0 context: {}",
+                                    creation_error
+                                )
+                            })
+                    },
+                    |window_builder, event_loop: &dyn EventLoopInterface| {
+                        let builder = ContextBuilder::new().with_vsync(true);
+                        // With latest Windows 10 and VmWare glutin's default for srgb produces surfaces that are always rendered black :(
+                        #[cfg(target_os = "windows")]
+                        let builder = builder.with_srgb(false);
+                        builder
+                            .build_windowed(window_builder, event_loop.event_loop_target())
+                            .map_err(|creation_error| {
+                                format!(
+                                    "could not create GlLatest context (with windows quirk): {}",
+                                    creation_error
+                                )
+                            })
+                    },
+                    |window_builder, event_loop: &dyn EventLoopInterface| {
+                        // Try again with glutin defaults
+                        ContextBuilder::new()
+                            .with_vsync(true)
+                            .build_windowed(window_builder, event_loop.event_loop_target())
+                            .map_err(|creation_error| {
+                                format!("could not create GlLatest context : {}", creation_error)
+                            })
+                    },
+                ];
 
-                // With latest Windows 10 and VmWare glutin's default for srgb produces surfaces that are always rendered black :(
-                #[cfg(target_os = "windows")]
-                let builder = builder.with_srgb(false);
+                let mut last_err = None;
+                for factory_fn in context_factory_fns {
+                    match factory_fn(window_builder.clone(), event_loop) {
+                        Ok(new_context) => {
+                            return new_context;
+                        }
+                        Err(e) => {
+                            last_err = Some(e);
+                        }
+                    }
+                }
 
-                builder.build_windowed(window_builder, event_loop.event_loop_target()).unwrap()
+                panic!("Failed to create OpenGL context: {}", last_err.unwrap())
             });
             let windowed_context = unsafe { windowed_context.make_current().unwrap() };
 
@@ -443,21 +488,14 @@ impl ItemRenderer for GLItemRenderer {
 
         let string = text.text();
         let string = string.as_str();
-        let font = text
-            .cached_rendering_data
-            .get_or_update(&self.graphics_window.graphics_cache, || {
-                Some(ItemGraphicsCacheEntry::Font(fonts::FONT_CACHE.with(|cache| {
-                    cache.borrow_mut().font(
-                        text.unresolved_font_request()
-                            .merge(&self.graphics_window.default_font_properties()),
-                        self.scale_factor,
-                        &text.text(),
-                    )
-                })))
-            })
-            .unwrap()
-            .as_font()
-            .clone();
+        let font = fonts::FONT_CACHE.with(|cache| {
+            cache.borrow_mut().font(
+                text.unresolved_font_request()
+                    .merge(&self.graphics_window.default_font_properties()),
+                self.scale_factor,
+                &text.text(),
+            )
+        });
 
         let paint = match self
             .brush_to_paint(text.color(), &mut rect_to_path(item_rect(text, self.scale_factor)))
@@ -489,22 +527,15 @@ impl ItemRenderer for GLItemRenderer {
             return;
         }
 
-        let font = text_input
-            .cached_rendering_data
-            .get_or_update(&self.graphics_window.graphics_cache, || {
-                Some(ItemGraphicsCacheEntry::Font(fonts::FONT_CACHE.with(|cache| {
-                    cache.borrow_mut().font(
-                        text_input
-                            .unresolved_font_request()
-                            .merge(&self.graphics_window.default_font_properties()),
-                        self.scale_factor,
-                        &text_input.text(),
-                    )
-                })))
-            })
-            .unwrap()
-            .as_font()
-            .clone();
+        let font = fonts::FONT_CACHE.with(|cache| {
+            cache.borrow_mut().font(
+                text_input
+                    .unresolved_font_request()
+                    .merge(&self.graphics_window.default_font_properties()),
+                self.scale_factor,
+                &text_input.text(),
+            )
+        });
 
         let paint = match self.brush_to_paint(
             text_input.color(),
@@ -516,7 +547,7 @@ impl ItemRenderer for GLItemRenderer {
 
         let (min_select, max_select) = text_input.selection_anchor_and_cursor();
         let cursor_pos = text_input.cursor_position();
-        let cursor_visible = cursor_pos >= 0 && text_input.cursor_visible();
+        let cursor_visible = cursor_pos >= 0 && text_input.cursor_visible() && text_input.enabled();
         let mut canvas = self.canvas.borrow_mut();
         let font_height = canvas.measure_font(paint).unwrap().height();
         let text = text_input.text();
@@ -977,7 +1008,6 @@ impl ItemRenderer for GLItemRenderer {
         let image_id = match cache_entry {
             Some(ItemGraphicsCacheEntry::Image(image)) => image.ensure_uploaded_to_gpu(self, None),
             Some(ItemGraphicsCacheEntry::ColorizedImage { .. }) => unreachable!(),
-            Some(ItemGraphicsCacheEntry::Font(_)) => unreachable!(),
             None => return,
         };
         let mut canvas = self.canvas.borrow_mut();
@@ -990,6 +1020,10 @@ impl ItemRenderer for GLItemRenderer {
         canvas.fill_path(&mut path, fill_paint);
     }
 
+    fn window(&self) -> WindowRc {
+        self.graphics_window.self_weak.upgrade().unwrap()
+    }
+
     fn as_any(&mut self) -> &mut dyn std::any::Any {
         self
     }
@@ -1101,7 +1135,7 @@ impl GLItemRenderer {
         });
 
         ItemGraphicsCacheEntry::ColorizedImage {
-            original_image: original_image.clone(),
+            _original_image: original_image.clone(),
             colorized_image: Rc::new(CachedImage::new_on_gpu(&self.canvas, colorized_image)),
         }
     }
@@ -1374,7 +1408,26 @@ pub mod native_widgets {}
 pub const HAS_NATIVE_STYLE: bool = false;
 pub const IS_AVAILABLE: bool = true;
 
-thread_local!(pub(crate) static CLIPBOARD : RefCell<copypasta::ClipboardContext> = std::cell::RefCell::new(copypasta::ClipboardContext::new().unwrap()));
+// TODO: We can't connect to the wayland clipboard yet because
+// it requires an external connection.
+cfg_if::cfg_if! {
+    if #[cfg(all(
+             unix,
+             not(any(
+                 target_os = "macos",
+                 target_os = "android",
+                 target_os = "ios",
+                 target_os = "emscripten"
+            )),
+            not(feature = "x11")
+        ))] {
+        type ClipboardBackend = copypasta::nop_clipboard::NopClipboardContext;
+    } else {
+        type ClipboardBackend = copypasta::ClipboardContext;
+    }
+}
+
+thread_local!(pub(crate) static CLIPBOARD : RefCell<ClipboardBackend> = std::cell::RefCell::new(ClipboardBackend::new().unwrap()));
 
 thread_local!(pub(crate) static IMAGE_CACHE: RefCell<images::ImageCache> = Default::default());
 
@@ -1402,7 +1455,7 @@ impl sixtyfps_corelib::backend::Backend for Backend {
 
     fn register_font_from_memory(
         &'static self,
-        data: &[u8],
+        data: &'static [u8],
     ) -> Result<(), Box<dyn std::error::Error>> {
         self::fonts::register_font_from_memory(data)
     }
